import datetime
from bisect import bisect

import xbmc
from xbmcgui import Dialog, DialogProgress, ListItem

from resources.lib.const import STRINGS, SETTINGS, LANG, quality_map, lang_code_gui, audio_channels, ROUTE
from resources.lib.gui import MediaItem
from resources.lib.storage.settings import settings
from resources.lib.utils.kodiutils import show_input, get_string, convert_size, make_table, \
    append_list_items_to_nested_list_items, convert_bitrate


class DialogRenderer:

    @staticmethod
    def search():
        search_value = show_input(get_string(30207))
        if search_value:
            return search_value

    @staticmethod
    def find_closest_resolution(height):
        keys = quality_map.keys()
        keys.sort()
        index = bisect(keys, height)
        return quality_map[keys[index]]

    @staticmethod
    def resolution_to_quality(video):
        height = video.get('height')
        quality = quality_map.get(height) or DialogRenderer.find_closest_resolution(height)
        if video['3d']:
            quality += ' 3D'
        return quality

    @staticmethod
    def choose_video_stream(streams):
        stream_labels = []
        streams.sort(key=lambda d: d.get('size', 0), reverse=settings.as_int(SETTINGS.FILE_SIZE_SORT))
        audio_info_list = []
        for stream in streams:
            # Fix audio string that begins with the comma.
            audio_info = []
            for audio in stream.get('audio'):
                lang_code = lang_code_gui.get(audio.get('language', '').lower()) or ''
                channels = audio.get('channels')
                channels = audio_channels.get(channels, channels)
                audio_info.append(STRINGS.AUDIO_INFO.format(audio.get('codec'),
                                                            format(channels, '.1f'),
                                                            lang_code))
            audio_info_list.append(' '.join(audio_info))
            video = stream.get('video')
            if len(video) > 0:
                video = video[0]
                quality = STRINGS.STREAM_TITLE_BRACKETS.format(
                    DialogRenderer.resolution_to_quality(video))
            else:
                quality = ''
                video = {}

            size = STRINGS.BOLD.format(convert_size(stream.get('size')))
            if video.get('duration'):
                bit_rate = stream.get('size') / video.get('duration') * 8
                duration = STRINGS.BOLD.format(
                    str(datetime.timedelta(seconds=int(video.get('duration'))))) if settings.as_bool(
                    SETTINGS.SHOW_DURATION) else ''
            else:
                bit_rate = 0
                duration = '-'
            bit_rate = STRINGS.STREAM_BITRATE_BRACKETS.format(
                convert_bitrate(bit_rate)) if settings.as_bool(
                SETTINGS.SHOW_BITRATE) else ''
            codec = STRINGS.STREAM_TITLE_BRACKETS.format(video.get('codec')) if settings.as_bool(
                SETTINGS.SHOW_CODEC) else ''

            stream_labels.append([quality, codec, size, duration, bit_rate])

        table = make_table(stream_labels)
        table = append_list_items_to_nested_list_items(table, audio_info_list)

        ret = Dialog().select(get_string(LANG.CHOOSE_STREAM), [STRINGS.TABLE_SPACES.join(item) for item in table])
        if ret < 0:
            return None
        return streams[ret]

    @staticmethod
    def keyboard(title, hidden=False):
        keyboard = xbmc.Keyboard('', title, hidden)
        keyboard.doModal()
        if keyboard.isConfirmed():
            return keyboard.getText().strip()
        else:
            return None

    @staticmethod
    def ok(heading, *args, **kwargs):
        return Dialog().ok(heading, *args, **kwargs)

    @staticmethod
    def ok_multi_line(heading, lines):
        return DialogRenderer.ok(heading, STRINGS.NEW_LINE.join(lines))

    @staticmethod
    def progress(heading, *args, **kwargs):
        d = DialogProgress()
        d.create(heading, *args, **kwargs)
        return d

    @staticmethod
    def select(heading, choices):
        return Dialog().select(heading, choices)

    @staticmethod
    def yesno(heading, *args, **kwargs):
        return Dialog().yesno(heading, *args, **kwargs)

    # @staticmethod
    # def select_stream(media_id, streams):
    #     stream = None
    #
    #     if len(streams) == 1:
    #         stream = streams[0]
    #     elif len(streams) > 1:
    #         force = self._router.args.has_key('force') and self._router.args['force'][0].lower() == 'true'
    #         if settings.get_setting_as_bool('auto_select_stream') and not force:
    #             preferred_quality = int(settings.get_settings(SETTINGS.STREAM_AUTOSELECT_PREFERRED_QUALITY))
    #             preferred_language = int(settings.get_settings(SETTINGS.STREAM_AUTOSELECT_PREFERRED_LANGUAGE))
    #             max_bitrate = settings.get_setting_as_int('max_bitrate') if settings.get_setting_as_bool('limit_bitrate') else 0
    #             avoid_vcodecs = (settings.get_settings('avoid_video_codecs') or '').split(',')
    #             preferred_channels = float(settings.get_settings('preferred_channels')) if settings.get_setting_as_bool('preferred_channels_enabled') else None
    #             stream = MediaListRenderer._autoselect_stream(
    #                 streams, preferred_quality, preferred_language, max_bitrate, avoid_vcodecs, preferred_channels)
    #
    #     if stream is None:
    #         stream = DialogRenderer.choose_video_stream(streams)
    #
    #     if stream is None:
    #         # Dialog cancel.
    #         set_resolved_url(self.handle)
    #         return False
    #
    #     logger.info('Got movie stream')
    #     self._on_stream_selected(stream['ident'])
