import re
from collections import OrderedDict

from resources.lib.api.api import API
from resources.lib.const import GENERAL, SETTINGS, STRINGS, lang_code_gui
from resources.lib.storage.settings import settings
from resources.lib.utils.kodiutils import strip_accents, translate_genres


class MediaInfoRenderer:
    class TITLE:
        @staticmethod
        def default(media, *args):
            info_labels = media.get('info_labels')
            title = info_labels.get('title')
            return MediaInfoRenderer.build_title(media, title, info_labels)

        @staticmethod
        def search(media, *args):
            info_labels = media.get('info_labels')
            titles = info_labels.get('parent_titles', []) or []
            titles.append(info_labels.get('title'))
            title = ' - '.join(titles)
            return MediaInfoRenderer.build_title(media, title, info_labels)

        @staticmethod
        def a_z(media, letters):
            info_labels = media.get('info_labels')
            languages, labels_dict = MediaInfoRenderer.get_language_priority_and_lang_map(media, True)
            title = MediaInfoRenderer.get_az_title(letters, languages, labels_dict) or info_labels.get('originaltitle')
            return MediaInfoRenderer.build_title(media, title, info_labels)

        @staticmethod
        def subtitles_search(title, info_labels):
            season = info_labels.get('season')
            episode = info_labels.get('episode')
            title = title.encode('utf-8')
            if season and episode:
                title = STRINGS.SEASON_EPISODE_TITLE.format(title, str(season).zfill(2), str(episode).zfill(2))
            elif season and not episode:
                title = STRINGS.SEASON_TITLE.format(title, str(season).zfill(2))
            elif not season and episode:
                title = STRINGS.EPISODE_TITLE.format(title, str(season).zfill(2), str(episode).zfill(2))
            return title

    @staticmethod
    def build_title(media, title, info_labels):
        title = STRINGS.TITLE_GENRE_YEAR.format(
            title=title.encode('utf-8'),
            genre=MediaInfoRenderer.process_genres(info_labels.get('genre')),
            year=info_labels.get('year')
        )
        audio = MediaInfoRenderer.preferred_lang_audio(media)
        if audio:
            title = STRINGS.TITLE_AUDIO_INFO.format(audio=lang_code_gui.get(audio), title=title)
        if not MediaInfoRenderer.stream_available(media) and media.get('children_count', 0) == 0:
            title = STRINGS.TITLE_STREAM_AVAILABLE.format(title=title)
        return title

    @staticmethod
    def merge_info_labels(media):
        translated_labels, art = MediaInfoRenderer.get_translated_labels(media)
        info_labels = media.get('info_labels')
        info_labels.update(translated_labels)
        return info_labels, art

    @staticmethod
    def stream_available(media):
        stream_count = media.get('available_streams', {}).get('count', 0)
        return stream_count > 0

    @staticmethod
    def preferred_lang_audio(media):
        langs = media.get('available_streams', {}).get('audio_languages')
        langs_settings = settings.get_languages(SETTINGS.PREFERRED_LANGUAGE, SETTINGS.FALLBACK_LANGUAGE)
        if langs:
            for lang_s in langs_settings:
                if lang_s and lang_s in langs:
                    return lang_s
        return ''

    @staticmethod
    def process_genres(genres):
        return ' / '.join(translate_genres(genres[:settings.as_int(SETTINGS.GENRES_COUNT)])).encode('utf-8')

    @staticmethod
    def get_az_title(letters, languages, labels_dict, when_missing=None):
        for lang_name in languages:
            value = labels_dict[lang_name].get('title')
            if value:
                stripped_value = re.sub('[^A-Za-z ]+', '', strip_accents(value))
                if stripped_value.lower().startswith(letters.replace(' ', '').lower()):
                    return value
        return when_missing

    @staticmethod
    def get_localized_item(languages, labels_dict, key, when_missing=None):
        for lang_name in languages:
            value = labels_dict[lang_name].get(key)
            if value:
                if isinstance(value, list):
                    for index, item in enumerate(value):
                        if not item:
                            for lang_name2 in languages:
                                value2 = labels_dict[lang_name2].get(key)[index]
                                if value2:
                                    value[index] = value2
                                    break
                return value

        return when_missing

    @staticmethod
    def get_localized_item_nested(languages, labels_dict, dict_key, key, when_missing='invalid-url'):
        for lang_name in languages:
            value = labels_dict[lang_name].get(dict_key, {}).get(key)
            if value:
                return value
        return when_missing

    @staticmethod
    def get_language_priority_and_lang_map(media, all_languages=False):
        labels = media.get('i18n_info_labels')
        original_lang = media.get('language')
        labels_dict = {}

        preferred = settings.get_select(SETTINGS.PREFERRED_LANGUAGE)
        fallback = settings.get_select(SETTINGS.FALLBACK_LANGUAGE)
        langs = [preferred]
        if fallback:
            langs.append(fallback)
        for item in labels:
            lang = item.get('lang')
            labels_dict[lang] = item
            if original_lang == lang:
                langs.append(original_lang)

        langs.append(GENERAL.DEFAULT_LANGUAGE)
        if all_languages:
            available_langs_other = set([v['lang'] for v in labels]) - {preferred, fallback}
            langs = langs + list(available_langs_other)

        return list(OrderedDict.fromkeys(langs)), labels_dict

    @staticmethod
    def get_translated_labels_all(media, new_dict):
        languages, labels_dict = MediaInfoRenderer.get_language_priority_and_lang_map(media, True)
        original_title = media.get('info_labels').get('originaltitle')
        if new_dict['title'] is None and original_title is None:
            new_dict['title'] = MediaInfoRenderer.get_localized_item(languages, labels_dict, 'title')
        elif new_dict['title'] is None and original_title:
            new_dict['title'] = original_title

        if new_dict['parent_titles']:
            for item in new_dict['parent_titles']:
                if not item:
                    new_dict['parent_titles'] = MediaInfoRenderer.get_localized_item(languages, labels_dict, 'parent_titles')
                    break
        return new_dict

    @staticmethod
    def get_translated_labels(media):
        languages, labels_dict = MediaInfoRenderer.get_language_priority_and_lang_map(media)
        new_dict = {}
        art = {}
        for label_name in ['title', 'plot', 'plotoutline', 'rating', 'votes', 'tagline', 'trailer', 'parent_titles']:
            new_dict[label_name] = MediaInfoRenderer.get_localized_item(languages, labels_dict, label_name)

        for label_name in ['poster', 'fanart', 'thumb', 'banner', 'clearart', 'clearlogo', 'landspace', 'icon']:
            art[label_name] = MediaInfoRenderer.get_localized_item_nested(languages, labels_dict, 'art', label_name)

        MediaInfoRenderer.get_translated_labels_all(media, new_dict)
        return new_dict, art
